import croniter
from datetime import datetime as dt

def handle_error(e, detail=""):
    """Generic error handler function.
    
    Parameters:
        e : Exception
            The exception object.
        detail : str
            Additional detail or message to include in the error log.
    
    Returns:
        None
    """
    print(f'Error: {str(e)}')
    if detail: print(f'detail: {detail}')


def get_next_cron_date(cron_expression:str, base_time:dt = dt.now()):
    """
    This function calculates the next date based on a cron expression.

    Parameters:
        cron_expression : str
            A string representing the cron expression.
        base_time : datetime
            A datetime object representing the base time to calculate the next date, if not provided, the current time will be used.

    Returns:
        str
    """
    try:
        cron = croniter(cron_expression, base_time)
        next_date = cron.get_next(dt.datetime)
        next_date = next_date.strftime('%Y-%m-%d %H:%M:%S')
        print(f'Based on the cron expression "{cron_expression}", the next invocation will be at {next_date}')
        return next_date
    except Exception as e:
        detail = f'Error getting next cron date: {str(e)}'
        handle_error(e,detail)


import boto3
import datetime as dt
from .utils import handle_error

def get_params(key:dict, table_name:str, params:list):
    '''This function retrieves specific parameters from a DynamoDB table item.
    
    Parameters:
        key : dict
            A dictionary where each key-value pair represents an attribute name and its value that will be used to identify the item in the DynamoDB table.
        table_name : str
            The name of the DynamoDB table where the item is stored.
        params : list
            A list of strings representing the names of the attributes that will be retrieved from the DynamoDB item.

    Return:
        dict
    '''
    try:
        dynamodb = boto3.resource('dynamodb')
        table = dynamodb.Table(table_name)
        response = table.get_item(Key=key)
        item = response['Item']
        return {param: item[param] for param in params}
    except Exception as e:
        detail = f'Error getting params from DynamoDB: {str(e)}'
        handle_error(e,detail)


def update_item(key:dict, table_name:str, update_dict:dict):
    """This function update data from an specific item on a DynamoDB table.
    
    Parameters:
        key : dict
            A dictionary where each key-value pair represents an attribute name and its value that will be used to identify the item in the DynamoDB table.
        table_name : str
            The name of the DynamoDB table where the item is stored.
        update_dict : dict
            A dictionary where each key-value pair represents an attribute name and its new value that will be updated in the DynamoDB item.
        
    Return:
        None
    """
    dynamodb = boto3.resource('dynamodb')
    dynamodb = dynamodb.Table(table_name)

    for param, value in update_dict.items():
        dynamodb.update_item(
            Key=key,
            UpdateExpression=f'SET {param} = :val',
            ExpressionAttributeValues={':val': value}
        )

    print(f'DynamoDB updated successfully - {dt.datetime.now()}')


import boto3
import json
from .utils import handle_error

def get_secret(secret_name:str, key_name:str=None):
    """This function retrieves a secret value from AWS Secrets Manager.

    Parameters:
        secret_name : str
            The name or ARN of the secret to retrieve.
        key_name : str
            The key name of the secret value to retrieve. If None, the entire secret will be returned.

    Return:
        Specific secret value (str) or the entire secret (dict).
    """
    try:
        session = boto3.session.Session()
        client = session.client(service_name='secretsmanager')
        
        get_secret_value_response = client.get_secret_value(SecretId=secret_name)
        secret = json.loads(get_secret_value_response['SecretString'])

        if key_name:
            value = secret.get(key_name)

            if value:
                print(f'Secret retrieved successfully: {secret_name} - {key_name}')
                return value
            else:
                print(f'key {key_name} not found in secret {secret_name}')
                return None
        
        else:
            print(f'Secret retrieved successfully: {secret_name}')
            return secret
            
    except Exception as e:
        detail = f'Error retrieving secret: {secret_name} - {key_name}'
        handle_error(e,detail)


import boto3
import os
import datetime as dt
from .utils import handle_error

def list_files(bucket_name:str, s3_path:str):
    """List all files at a given S3 path.
    
    Parameters:
        bucket_name : str
            The name of the S3 bucket.
        s3_path : str
            The path within the S3 bucket.
    
    Returns:
        list: A list of file keys (paths) in the specified S3 path.
    """
    try:
        s3_client = boto3.client('s3')
        paginator = s3_client.get_paginator('list_objects_v2')
        result = []

        for page in paginator.paginate(Bucket=bucket_name, Prefix=s3_path):
            if 'Contents' in page:
                for obj in page['Contents']:
                    result.append(obj['Key']) if (obj['Key']).endswith('.csv') else None

        return result
    except Exception as e:
        detail = f'Error listing S3 files: {str(e)}'
        handle_error(e,detail)


def get_files(bucket:str, keys:list=None):
    """This function retrieves a file from an S3 bucket.
    
    Parameters:
        bucket : str
            The name of the S3 bucket where the file is stored.
        keys : list
            A list of keys of the files in the S3 bucket that needs to be retrieved.
    
    Return:
        list
    """
    try:
        s3 = boto3.client('s3')
        local_files = []
        for key in keys:
            local_file = '/tmp/' + os.path.basename(key)
            s3.download_file(bucket, key, local_file)
            local_files.append(local_file)
        if local_files: print(f'Files retrieved from S3 successfully - {dt.datetime.now()}')
        return local_files
    except Exception as e:
        detail = f'Error retrieving file from S3: {str(e)}'
        handle_error(e,detail)
    

def delete_files(bucket:str, keys:str):
    """This function deletes a file from an S3 bucket.
        
        Parameters:
            bucket : str
                The name of the S3 bucket where the file is stored.
            keys : list
                A list of keys of the files in the S3 bucket that needs to be deleted.
            
        Return:
            None
        """
    try:
        s3 = boto3.client('s3')
        for key in keys:
            s3.delete_object(Bucket=bucket, Key=key)
            print(f'File deleted from S3 successfully - {key} - {dt.datetime.now()}')
    except Exception as e:
        detail = f'Error deleting file from S3: {str(e)}'
        handle_error(e,detail)


from setuptools import setup, find_packages

setup(
    name='cntcar_aws',
    version='0.0.9',
    description='Python library for AWS services used in ConectCar projects',
    long_description_content_type="text/markdown",
    long_description='README.md',
    author='Marcos Lemes',
    classifiers=[
                "Development Status :: 3 - Alpha",
                "Programming Language :: Python :: 3",
                "Programming Language :: Python :: 3.8"],
    packages=find_packages(),
    install_requires=[
        'boto3',
        'croniter'
    ]
)